"""
Test models
"""

# Standard Library
from unittest.mock import Mock, patch

# Third Party
from requests.exceptions import HTTPError

# Django
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.db import IntegrityError
from django.test import TestCase

# AA Discord Announcements
from aa_discord_announcements.models import (
    General,
    PingTarget,
    Webhook,
    _get_discord_group_info,
)


class TestGetDiscordGroupInfo(TestCase):
    """
    Test _get_discord_group_info function
    """

    @patch(
        "aa_discord_announcements.models.discord_service_installed", return_value=True
    )
    @patch("aa_discord_announcements.models.DiscordUser.objects.group_to_role")
    def test_returns_discord_group_info(
        self, mock_group_to_role, mock_discord_service_installed
    ):
        """
        Test if _get_discord_group_info returns the correct Discord group info

        :param mock_group_to_role:
        :type mock_group_to_role:
        :param mock_discord_service_installed:
        :type mock_discord_service_installed:
        :return:
        :rtype:
        """

        mock_group = Mock(spec=Group)
        mock_group_to_role.return_value = {"id": "123456789"}
        result = _get_discord_group_info(mock_group)

        self.assertEqual(result, {"id": "123456789"})

    @patch(
        "aa_discord_announcements.models.discord_service_installed", return_value=False
    )
    def test_raises_error_if_discord_service_not_installed(
        self, mock_discord_service_installed
    ):
        """
        Test if _get_discord_group_info raises a ValidationError if the Discord service is not installed

        :param mock_discord_service_installed:
        :type mock_discord_service_installed:
        :return:
        :rtype:
        """

        mock_group = Mock(spec=Group)

        with self.assertRaises(ValidationError):
            _get_discord_group_info(mock_group)

    @patch(
        "aa_discord_announcements.models.discord_service_installed", return_value=True
    )
    @patch(
        "aa_discord_announcements.models.DiscordUser.objects.group_to_role",
        side_effect=HTTPError,
    )
    def test_raises_error_if_http_error_occurs(
        self, mock_group_to_role, mock_discord_service_installed
    ):
        """
        Test if _get_discord_group_info raises a ValidationError if an HTTPError occurs

        :param mock_group_to_role:
        :type mock_group_to_role:
        :param mock_discord_service_installed:
        :type mock_discord_service_installed:
        :return:
        :rtype:
        """

        mock_group = Mock(spec=Group)

        with self.assertRaises(ValidationError):
            _get_discord_group_info(mock_group)

    @patch(
        "aa_discord_announcements.models.discord_service_installed", return_value=True
    )
    @patch(
        "aa_discord_announcements.models.DiscordUser.objects.group_to_role",
        return_value=None,
    )
    def test_raises_error_if_group_not_synced(
        self, mock_group_to_role, mock_discord_service_installed
    ):
        """
        Test if _get_discord_group_info raises a ValidationError if the group is not synced to Discord

        :param mock_group_to_role:
        :type mock_group_to_role:
        :param mock_discord_service_installed:
        :type mock_discord_service_installed:
        :return:
        :rtype:
        """

        mock_group = Mock(spec=Group)

        with self.assertRaises(ValidationError):
            _get_discord_group_info(mock_group)


class TestGeneralModel(TestCase):
    """
    Test General model
    """

    def test_can_access_app_permission(self):
        """
        Test if the permission 'basic_access' is in the permissions tuple

        :return:
        :rtype:
        """

        general = General()

        self.assertIn("basic_access", dict(general._meta.permissions))

    def test_is_not_managed(self):
        """
        Test if the model is not managed by Django

        :return:
        :rtype:
        """

        general = General()

        self.assertFalse(general._meta.managed)

    def test_has_no_default_permissions(self):
        """
        Test if the model has no default permissions

        :return:
        :rtype:
        """

        general = General()

        self.assertEqual(general._meta.default_permissions, ())


class TestPingTargetModel(TestCase):
    """
    Test the PingTarget model
    """

    @patch(
        "aa_discord_announcements.models.discord_service_installed", return_value=True
    )
    @patch("aa_discord_announcements.models._get_discord_group_info")
    def test_saves_with_valid_discord_group(
        self, mock_get_discord_group_info, mock_discord_service_installed
    ):
        """
        Test if the PingTarget model saves with a valid Discord group

        :param mock_get_discord_group_info:
        :type mock_get_discord_group_info:
        :param mock_discord_service_installed:
        :type mock_discord_service_installed:
        :return:
        :rtype:
        """

        mock_get_discord_group_info.return_value = {"id": "123456789"}
        group = Group.objects.create(name="Test Group")
        ping_target = PingTarget(name=group)
        ping_target.save()

        self.assertEqual(ping_target.discord_id, "123456789")

    @patch(
        "aa_discord_announcements.models.discord_service_installed", return_value=False
    )
    def test_raises_error_if_discord_service_not_installed_on_save(
        self, mock_discord_service_installed
    ):
        """
        Test if the PingTarget model raises a ValidationError if the Discord service is not installed

        :param mock_discord_service_installed:
        :type mock_discord_service_installed:
        :return:
        :rtype:
        """

        group = Group.objects.create(name="Test Group")
        ping_target = PingTarget(name=group)

        with self.assertRaises(ValidationError):
            ping_target.save()

    @patch(
        "aa_discord_announcements.models.discord_service_installed", return_value=True
    )
    @patch(
        "aa_discord_announcements.models._get_discord_group_info",
        side_effect=ValidationError("This group has not been synced to Discord yet."),
    )
    def test_raises_error_if_discord_group_not_synced_on_save(
        self, mock_get_discord_group_info, mock_discord_service_installed
    ):
        """
        Test if the PingTarget model raises a ValidationError if the Discord group is not synced

        :param mock_get_discord_group_info:
        :type mock_get_discord_group_info:
        :param mock_discord_service_installed:
        :type mock_discord_service_installed:
        :return:
        :rtype:
        """

        group = Group.objects.create(name="Test Group")
        ping_target = PingTarget(name=group)
        with self.assertRaises(ValidationError):
            ping_target.save()

    @patch(
        "aa_discord_announcements.models.discord_service_installed", return_value=True
    )
    @patch("aa_discord_announcements.models._get_discord_group_info")
    def test_cleans_with_valid_discord_group(
        self, mock_get_discord_group_info, mock_discord_service_installed
    ):
        """
        Test if the PingTarget model cleans with a valid Discord group

        :param mock_get_discord_group_info:
        :type mock_get_discord_group_info:
        :param mock_discord_service_installed:
        :type mock_discord_service_installed:
        :return:
        :rtype:
        """

        mock_get_discord_group_info.return_value = {"id": "123456789"}
        group = Group.objects.create(name="Test Group")
        ping_target = PingTarget(name=group)
        ping_target.clean()

    @patch(
        "aa_discord_announcements.models.discord_service_installed", return_value=False
    )
    def test_raises_error_if_discord_service_not_installed_on_clean(
        self, mock_discord_service_installed
    ):
        """
        Test if the PingTarget model raises a ValidationError if the Discord service is not installed when cleaning the model

        :param mock_discord_service_installed:
        :type mock_discord_service_installed:
        :return:
        :rtype:
        """

        group = Group.objects.create(name="Test Group")
        ping_target = PingTarget(name=group)

        with self.assertRaises(ValidationError):
            ping_target.clean()

    @patch(
        "aa_discord_announcements.models.discord_service_installed", return_value=True
    )
    @patch(
        "aa_discord_announcements.models._get_discord_group_info",
        side_effect=ValidationError("This group has not been synced to Discord yet."),
    )
    def test_raises_error_if_discord_group_not_synced_on_clean(
        self, mock_get_discord_group_info, mock_discord_service_installed
    ):
        """
        Test if the PingTarget model raises a ValidationError if the Discord group is not synced when cleaning the model

        :param mock_get_discord_group_info:
        :type mock_get_discord_group_info:
        :param mock_discord_service_installed:
        :type mock_discord_service_installed:
        :return:
        :rtype:
        """

        group = Group.objects.create(name="Test Group")
        ping_target = PingTarget(name=group)

        with self.assertRaises(ValidationError):
            ping_target.clean()

    def test_should_return_ping_target_model_string_name(self):
        """
        Test should return the PingTarget model string name

        :return:
        :rtype:
        """

        group = Group.objects.create(name="Test Group")
        ping_target = PingTarget(name=group)

        self.assertEqual(first=str(ping_target), second=group.name)


class TestWebhookModel(TestCase):
    """
    Test the Webhook model
    """

    def test_saves_with_valid_data(self):
        """
        Test if the Webhook model saves with valid data

        :return:
        :rtype:
        """

        webhook = Webhook.objects.create(
            name="Test Channel", url="https://discord.com/api/webhooks/123456/abcdef"
        )
        self.assertEqual(webhook.name, "Test Channel")
        self.assertEqual(webhook.url, "https://discord.com/api/webhooks/123456/abcdef")

    def test_raises_error_with_invalid_url(self):
        """
        Test if the Webhook model raises a ValidationError with an invalid URL

        :return:
        :rtype:
        """

        webhook = Webhook(name="Test Channel", url="invalid_url")

        with self.assertRaises(ValidationError):
            webhook.clean()

    def test_cleans_with_valid_url(self):
        """
        Test if the Webhook model cleans with a valid URL

        :return:
        :rtype:
        """

        webhook = Webhook(
            name="Test Channel", url="https://discord.com/api/webhooks/123456/abcdef"
        )
        webhook.clean()

        self.assertEqual(webhook.url, "https://discord.com/api/webhooks/123456/abcdef")

    def test_webhook_url_is_unique(self):
        """
        Test if the Webhook model raises an IntegrityError if the URL is not unique

        :return:
        :rtype:
        """

        Webhook.objects.create(
            name="Test Channel 1",
            url="https://discord.com/api/webhooks/123456/abcdef",
        )

        with self.assertRaises(IntegrityError):
            Webhook.objects.create(
                name="Test Channel 2",
                url="https://discord.com/api/webhooks/123456/abcdef",
            )

    def test_saves_with_restricted_groups(self):
        """
        Test if the Webhook model saves with restricted groups

        :return:
        :rtype:
        """

        group = Group.objects.create(name="Test Group")
        webhook = Webhook.objects.create(
            name="Test Channel", url="https://discord.com/api/webhooks/123456/abcdef"
        )
        webhook.restricted_to_group.add(group)

        self.assertIn(group, webhook.restricted_to_group.all())

    def test_saves_with_notes(self):
        """
        Test if the Webhook model saves with notes

        :return:
        :rtype:
        """

        webhook = Webhook.objects.create(
            name="Test Channel",
            url="https://discord.com/api/webhooks/123456/abcdef",
            notes="Test notes",
        )

        self.assertEqual(webhook.notes, "Test notes")

    def test_saves_with_is_enabled(self):
        """
        Test if the Webhook model saves with is_enabled

        :return:
        :rtype:
        """

        webhook = Webhook.objects.create(
            name="Test Channel",
            url="https://discord.com/api/webhooks/123456/abcdef",
            is_enabled=False,
        )

        self.assertFalse(webhook.is_enabled)

    def test_should_return_webhook_model_string_name(self):
        """
        Test should return the Webhook model string name

        :return:
        :rtype:
        """

        webhook = Webhook(
            name="Test Channel",
            url="https://discord.com/api/webhooks/123456/abcdef",
        )

        webhook.save()

        self.assertEqual(first=str(webhook), second="Test Channel")



"""
Test ajax calls
"""

# Standard Library
import json
from http import HTTPStatus
from unittest.mock import patch

# Django
from django.contrib.auth.models import Group
from django.test import RequestFactory, TestCase
from django.urls import reverse

# AA Discord Announcements
from aa_discord_announcements.tests.utils import create_fake_user
from aa_discord_announcements.views import ajax_create_announcement


class TestAjaxCalls(TestCase):
    """
    Test access to ajax calls
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()

        cls.group = Group.objects.create(name="Superhero")

        # User cannot access aa_discord_announcements
        cls.user_1001 = create_fake_user(
            character_id=1001, character_name="Peter Parker"
        )

        # User can access aa_discord_announcements
        cls.user_1002 = create_fake_user(
            character_id=1002,
            character_name="Bruce Wayne",
            permissions=["aa_discord_announcements.basic_access"],
        )

    def setUp(self):
        """
        Setup
        """

        self.factory = RequestFactory()

    def test_ajax_get_announcement_targets_no_access(self):
        """
        Test ajax call to get announcement targets available for the current user without access to it

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1001)

        # when
        res = self.client.get(
            path=reverse(
                viewname="aa_discord_announcements:ajax_get_announcement_targets"
            )
        )

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)

    def test_ajax_get_announcement_targets_general(self):
        """
        Test ajax call to get announcement targets available for the current user

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        res = self.client.get(
            path=reverse(
                viewname="aa_discord_announcements:ajax_get_announcement_targets"
            )
        )

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.OK)

    def test_ajax_get_webhooks_no_access(self):
        """
        Test ajax call to get webhooks available for the current user without access to it

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1001)

        # when
        res = self.client.get(
            path=reverse(viewname="aa_discord_announcements:ajax_get_webhooks")
        )

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)

    def test_ajax_get_webhooks_general(self):
        """
        Test ajax call to get webhooks available for the current user

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        res = self.client.get(
            path=reverse(viewname="aa_discord_announcements:ajax_get_webhooks")
        )

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.OK)

    def test_ajax_create_announcement_no_access(self):
        """
        Test ajax call to create an announcement is not available for a user without access to it

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1001)

        # when
        res = self.client.get(
            path=reverse(viewname="aa_discord_announcements:ajax_create_announcement")
        )

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)

    def test_ajax_create_announcement_general(self):
        """
        Test ajax call to create an announcement is available for the current user

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        res = self.client.get(
            path=reverse(viewname="aa_discord_announcements:ajax_create_announcement")
        )

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.OK)

    @patch("aa_discord_announcements.views.get_announcement_context_from_form_data")
    @patch("aa_discord_announcements.views.send_to_discord_webhook")
    def test_creates_announcement_successfully_with_webhook(
        self, mock_send_to_discord_webhook, mock_get_announcement_context
    ):
        """
        Test ajax call to create an announcement is successful with a webhook

        :param mock_send_to_discord_webhook:
        :type mock_send_to_discord_webhook:
        :param mock_get_announcement_context:
        :type mock_get_announcement_context:
        :return:
        :rtype:
        """

        mock_get_announcement_context.return_value = {
            "announcement_target": {
                "group_id": None,
                "group_name": None,
                "at_mention": "@here",
            },
            "announcement_channel": {"webhook": True},
            "announcement_text": "Borg to fight!",
        }

        self.client.force_login(user=self.user_1002)

        form_data = json.dumps(
            {
                "announcement_target": "@here",
                "announcement_channel": "1",
                "announcement_text": "Borg to fight!",
            }
        )
        response = self.client.post(
            path=reverse(viewname="aa_discord_announcements:ajax_create_announcement"),
            data=form_data,
            content_type="application/json",
        )

        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)
        self.assertTemplateUsed(
            response=response,
            template_name="aa_discord_announcements/partials/announcement/copy-paste-text.html",
        )
        self.assertContains(response=response, text="@here")
        self.assertContains(response=response, text="Borg to fight!")

        mock_send_to_discord_webhook.assert_called_once()

    @patch("aa_discord_announcements.views.get_announcement_context_from_form_data")
    @patch("aa_discord_announcements.views.send_to_discord_webhook")
    def test_creates_announcement_successfully_without_webhook(
        self, mock_send_to_discord_webhook, mock_get_announcement_context
    ):
        """
        Test ajax call to create an announcement is successful without a webhook

        :param mock_send_to_discord_webhook:
        :type mock_send_to_discord_webhook:
        :param mock_get_announcement_context:
        :type mock_get_announcement_context:
        :return:
        :rtype:
        """

        mock_get_announcement_context.return_value = {
            "announcement_target": {
                "group_id": None,
                "group_name": None,
                "at_mention": "@here",
            },
            "announcement_channel": {"webhook": False},
            "announcement_text": "Borg to fight!",
        }

        self.client.force_login(user=self.user_1002)

        form_data = json.dumps(
            {
                "announcement_target": "@here",
                "announcement_channel": "1",
                "announcement_text": "Borg to fight!",
            }
        )
        response = self.client.post(
            path=reverse(viewname="aa_discord_announcements:ajax_create_announcement"),
            data=form_data,
            content_type="application/json",
        )

        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)
        self.assertTemplateUsed(
            response=response,
            template_name="aa_discord_announcements/partials/announcement/copy-paste-text.html",
        )
        self.assertContains(response=response, text="@here")
        self.assertContains(response=response, text="Borg to fight!")

        mock_send_to_discord_webhook.assert_not_called()

    def test_form_invalid_returns_error(self):
        """
        Test ajax call to create an announcement returns an error if the form is invalid

        :return:
        :rtype:
        """

        request = self.factory.post(
            path=reverse(viewname="aa_discord_announcements:ajax_create_announcement"),
            data=json.dumps(
                {
                    "announcement_target": "800432143549333504",
                    "announcement_channel": "1",
                }
            ),
            content_type="application/json",
        )
        request.user = self.user_1002

        response = ajax_create_announcement(request)

        self.assertEqual(response.status_code, 200)
        self.assertFalse(json.loads(response.content)["success"])
        self.assertEqual(
            json.loads(response.content)["message"],
            "Form invalid. Please check your input.",
        )

    def no_form_data_submitted_returns_error(self):
        """
        Test ajax call to create an announcement returns an error if no form data is submitted

        :return:
        :rtype:
        """

        request = self.factory.post(
            path=reverse(viewname="aa_discord_announcements:ajax_create_announcement"),
            content_type="application/json",
        )
        request.user = self.user_1002

        response = ajax_create_announcement(request)

        self.assertEqual(response.status_code, 200)
        self.assertFalse(json.loads(response.content)["success"])
        self.assertEqual(
            json.loads(response.content)["message"], "No form data submitted."
        )



"""
Test the settings
"""

# Django
from django.test import TestCase, modify_settings, override_settings

# AA Discord Announcements
from aa_discord_announcements.app_settings import (
    debug_enabled,
    discord_service_installed,
)


class TestSettings(TestCase):
    """
    Test the settings
    """

    @modify_settings(INSTALLED_APPS={"append": "allianceauth.services.modules.discord"})
    def test_discord_service_installed_should_return_true(self) -> None:
        """
        Test discord_service_installed should return True with discord service enabled

        :return:
        :rtype:
        """

        self.assertTrue(expr=discord_service_installed())

    @modify_settings(INSTALLED_APPS={"remove": "allianceauth.services.modules.discord"})
    def test_discord_service_installed_should_return_false(self) -> None:
        """
        Test discord_service_installed should return False without discord service enabled

        :return:
        :rtype:
        """

        self.assertFalse(expr=discord_service_installed())

    @override_settings(DEBUG=True)
    def test_debug_enabled_with_debug_true(self) -> None:
        """
        Test debug_enabled with DEBUG = True

        :return:
        :rtype:
        """

        self.assertTrue(debug_enabled())

    @override_settings(DEBUG=False)
    def test_debug_enabled_with_debug_false(self) -> None:
        """
        Test debug_enabled with DEBUG = False

        :return:
        :rtype:
        """

        self.assertFalse(debug_enabled())



# Generated by Django 4.0.10 on 2023-06-19 10:01

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("aa_discord_announcements", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="pingtarget",
            name="discord_id",
            field=models.CharField(
                blank=True,
                help_text="ID of the Discord role to ping",
                max_length=255,
                unique=True,
                verbose_name="Discord ID",
            ),
        ),
        migrations.AlterField(
            model_name="pingtarget",
            name="is_enabled",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text="Whether this ping target is enabled or not",
                verbose_name="Is enabled",
            ),
        ),
        migrations.AlterField(
            model_name="pingtarget",
            name="name",
            field=models.OneToOneField(
                help_text="Name of the Discord role to ping. (Note: This must be an Auth group that is synced to Discord.)",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="discord_announcement_pingtarget",
                to="auth.group",
                verbose_name="Group name",
            ),
        ),
        migrations.AlterField(
            model_name="pingtarget",
            name="notes",
            field=models.TextField(
                blank=True,
                default="",
                help_text="You can add notes about this configuration here if you want",
                verbose_name="Notes",
            ),
        ),
        migrations.AlterField(
            model_name="pingtarget",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict ping rights to the following group(s) …",
                related_name="discord_announcement_pingtarget_required_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="is_enabled",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text="Whether this webhook is active or not",
                verbose_name="Is enabled",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="name",
            field=models.CharField(
                help_text="Name of the channel this webhook posts to",
                max_length=255,
                unique=True,
                verbose_name="Discord channel",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="notes",
            field=models.TextField(
                blank=True,
                default="",
                help_text="You can add notes about this webhook here if you want",
                verbose_name="Notes",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict ping rights to the following group(s) …",
                related_name="discord_announcement_webhook_required_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="url",
            field=models.CharField(
                help_text="URL of this webhook, e.g. https://discord.com/api/webhooks/123456/abcdef",
                max_length=255,
                unique=True,
                verbose_name="Webhook URL",
            ),
        ),
    ]



# Generated by Django 4.0.10 on 2023-06-28 14:14

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("aa_discord_announcements", "0002_translation_updates"),
    ]

    operations = [
        migrations.AlterField(
            model_name="pingtarget",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict ping rights to the following groups …",
                related_name="discord_announcement_pingtarget_required_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict ping rights to the following groups …",
                related_name="discord_announcement_webhook_required_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
    ]



# Generated by Django 4.0.10 on 2023-09-22 17:41

# Django
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("aa_discord_announcements", "0003_fix_pseudo_plural_in_help_text"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="pingtarget",
            options={
                "default_permissions": (),
                "verbose_name": "Discord ping target",
                "verbose_name_plural": "Discord ping targets",
            },
        ),
    ]



"""
Handling Discord webhooks
"""

# Third Party
from dhooks_lite import UserAgent, Webhook

# Django
from django.contrib.auth.models import User
from django.utils import dateformat, timezone

# AA Discord Announcements
from aa_discord_announcements import __version__
from aa_discord_announcements.constants import APP_NAME, GITHUB_URL
from aa_discord_announcements.helper.announcement_context import (
    get_webhook_announcement_context,
)


def get_user_agent() -> UserAgent:
    """
    Set the user agent for dhooks_lite

    :return: User agent
    :rtype: UserAgent
    """

    return UserAgent(APP_NAME, GITHUB_URL, __version__)


def send_to_discord_webhook(announcement_context: dict, user: User) -> None:
    """
    Send the announcement to a Discord webhook

    :param announcement_context: Announcement context
    :type announcement_context: dict
    :param user: User sending the announcement
    :type user: User
    :return: None
    :rtype: None
    """

    discord_webhook = Webhook(
        url=announcement_context["announcement_channel"]["webhook"],
        user_agent=get_user_agent(),
    )
    message_body = get_webhook_announcement_context(
        announcement_context=announcement_context
    )["content"]
    author_eve_name = user.profile.main_character.character_name

    message_to_send = (
        f"{message_body}\n\n"
        f"-# _Sent by {author_eve_name} @ {dateformat.format(value=timezone.now(), format_string='Y-m-d H:i')} (EVE time)_"
    )

    discord_webhook.execute(content=message_to_send, wait_for_response=True)



"""
Hook into AA
"""

# Alliance Auth
from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook

# AA Discord Announcements
from aa_discord_announcements import __title__, urls


class AaDiscordAnnouncementsMenuItem(
    MenuItemHook
):  # pylint: disable=too-few-public-methods
    """
    This class ensures only authorized users will see the menu entry
    """

    def __init__(self):
        # setup menu entry for sidebar
        MenuItemHook.__init__(
            self,
            text=__title__,
            classes="fa-regular fa-bell",
            url_name="aa_discord_announcements:index",
            navactive=["aa_discord_announcements:"],
        )

    def render(self, request):
        """
        Check if the user has the permission to view this app
        :param request:
        :return:
        """

        return (
            MenuItemHook.render(self, request=request)
            if request.user.has_perm(perm="aa_discord_announcements.basic_access")
            else ""
        )


@hooks.register("menu_item_hook")
def register_menu():
    """
    Register our menu item
    :return:
    """

    return AaDiscordAnnouncementsMenuItem()


@hooks.register("url_hook")
def register_urls():
    """
    Register our base url
    :return:
    """

    return UrlHook(
        urls=urls,
        namespace="aa_discord_announcements",
        base_url=r"^discord-announcements/",
    )



"""
The views
"""

# Standard Library
import json

# Django
from django.contrib.auth.decorators import login_required, permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.db.models import Q
from django.http import HttpResponse
from django.shortcuts import render
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag
from app_utils.urls import site_absolute_url

# AA Discord Announcements
from aa_discord_announcements import __title__
from aa_discord_announcements.forms import AnnouncementForm
from aa_discord_announcements.helper.announcement_context import (
    get_announcement_context_from_form_data,
)
from aa_discord_announcements.helper.discord_webhook import send_to_discord_webhook
from aa_discord_announcements.models import PingTarget, Webhook

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


@login_required
@permission_required(perm="aa_discord_announcements.basic_access")
def index(request: WSGIRequest) -> HttpResponse:
    """
    Index view
    """

    logger.info(msg=f"Discord Announcements view called by user {request.user}")

    context = {
        "title": __title__,
        "webhooks_configured": Webhook.objects.filter(
            Q(restricted_to_group__in=request.user.groups.all())
            | Q(restricted_to_group__isnull=True),
            is_enabled=True,
        ).exists(),
        "site_url": site_absolute_url(),
        "main_character": request.user.profile.main_character,
        "form": AnnouncementForm,
    }

    return render(
        request=request,
        template_name="aa_discord_announcements/index.html",
        context=context,
    )


@login_required
@permission_required(perm="aa_discord_announcements.basic_access")
def ajax_get_announcement_targets(request: WSGIRequest) -> HttpResponse:
    """
    Get announcement targets for the current user
    :param request:
    :return:
    """

    logger.info(msg=f"Getting announcement targets for user {request.user}")

    additional_discord_announcement_targets = (
        PingTarget.objects.filter(
            Q(restricted_to_group__in=request.user.groups.all())
            | Q(restricted_to_group__isnull=True),
            is_enabled=True,
        )
        .distinct()
        .order_by("name")
    )

    return render(
        request=request,
        template_name="aa_discord_announcements/partials/form/segments/announcement-targets.html",
        context={"announcement_targets": additional_discord_announcement_targets},
    )


@login_required
@permission_required(perm="aa_discord_announcements.basic_access")
def ajax_get_webhooks(request: WSGIRequest) -> HttpResponse:
    """
    Get webhooks for the current user
    :param request:
    :return:
    """

    logger.info(msg=f"Getting webhooks for user {request.user}")

    webhooks = (
        Webhook.objects.filter(
            Q(restricted_to_group__in=request.user.groups.all())
            | Q(restricted_to_group__isnull=True),
            is_enabled=True,
        ).distinct()
        # .order_by("type", "name")
    )

    return render(
        request=request,
        template_name="aa_discord_announcements/partials/form/segments/announcement-channel.html",
        context={"webhooks": webhooks},
    )


@login_required
@permission_required(perm="aa_discord_announcements.basic_access")
def ajax_create_announcement(request: WSGIRequest) -> HttpResponse:
    """
    Create the announcement

    :param request:
    :return:
    """

    context = {}

    if request.method == "POST":
        form = AnnouncementForm(data=json.loads(request.body))

        if form.is_valid():
            logger.info(msg="Discord announcement received")

            logger.debug(msg=f"Announcement form data: {form.cleaned_data}")

            # Get ping context
            announcement_context = get_announcement_context_from_form_data(
                form_data=form.cleaned_data
            )

            logger.debug(msg=f"Announcement context: {announcement_context}")

            # If we have a Discord webhook, ping it
            if announcement_context["announcement_channel"]["webhook"]:
                send_to_discord_webhook(
                    announcement_context=announcement_context, user=request.user
                )

            logger.info(msg=f"Discord announcement created by user {request.user}")

            context["announcement_context"] = render_to_string(
                template_name="aa_discord_announcements/partials/announcement/copy-paste-text.html",
                context=announcement_context,
            )
            context["success"] = True
        else:
            context["message"] = str(_("Form invalid. Please check your input."))
            context["success"] = False
    else:
        context["message"] = str(_("No form data submitted."))
        context["success"] = False

    return HttpResponse(content=json.dumps(context), content_type="application/json")



# Standard Library
import re
from unittest.mock import Mock, patch

# Django
from django.contrib.auth.models import Group, User
from django.test import TestCase

# AA Discord Announcements
from aa_discord_announcements.helper.announcement_context import (
    get_announcement_context_from_form_data,
    get_webhook_announcement_context,
)
from aa_discord_announcements.helper.discord_webhook import send_to_discord_webhook
from aa_discord_announcements.models import PingTarget, Webhook


class TestAnnouncementContext(TestCase):
    """
    Test the get_announcement_context_from_form_data function
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()

        cls.group = Group.objects.create(name="Superhero")

    @patch("aa_discord_announcements.models.PingTarget.objects.get")
    @patch("aa_discord_announcements.models.Webhook.objects.get")
    def test_gets_announcement_context_with_here_mention(
        self, mock_get_webhook, mock_get_ping_target
    ):
        """
        Test the get_announcement_context_from_form_data function with @here mention

        :param mock_get_webhook:
        :type mock_get_webhook:
        :param mock_get_ping_target:
        :type mock_get_ping_target:
        :return:
        :rtype:
        """

        form_data = {
            "announcement_target": "@here",
            "announcement_channel": 1,
            "announcement_text": "Test announcement",
        }

        mock_get_webhook.return_value.url = "http://example.com/webhook"
        context = get_announcement_context_from_form_data(form_data)

        self.assertEqual(context["announcement_target"]["at_mention"], "@here")
        self.assertEqual(
            context["announcement_channel"]["webhook"], "http://example.com/webhook"
        )
        self.assertEqual(context["announcement_text"], "Test announcement")

    @patch("aa_discord_announcements.models.PingTarget.objects.get")
    @patch("aa_discord_announcements.models.Webhook.objects.get")
    def test_gets_announcement_context_with_custom_target(
        self, mock_get_webhook, mock_get_ping_target
    ):
        """
        Test the get_announcement_context_from_form_data function with a custom ping target

        :param mock_get_webhook:
        :type mock_get_webhook:
        :param mock_get_ping_target:
        :type mock_get_ping_target:
        :return:
        :rtype:
        """

        form_data = {
            "announcement_target": self.group,
            "announcement_channel": 1,
            "announcement_text": "Test announcement",
        }

        mock_get_ping_target.return_value = PingTarget(
            discord_id="123456789", name=self.group
        )
        mock_get_webhook.return_value.url = "http://example.com/webhook"

        context = get_announcement_context_from_form_data(form_data)

        self.assertEqual(context["announcement_target"]["group_id"], 123456789)
        self.assertEqual(context["announcement_target"]["group_name"], self.group.name)
        self.assertEqual(
            context["announcement_target"]["at_mention"], f"@{self.group.name}"
        )
        self.assertEqual(
            context["announcement_channel"]["webhook"], "http://example.com/webhook"
        )
        self.assertEqual(context["announcement_text"], "Test announcement")

    @patch("aa_discord_announcements.models.PingTarget.objects.get")
    @patch("aa_discord_announcements.models.Webhook.objects.get")
    def test_handles_nonexistent_custom_target(
        self, mock_get_webhook, mock_get_ping_target
    ):
        """
        Test the get_announcement_context_from_form_data function with a nonexistent custom target

        :param mock_get_webhook:
        :type mock_get_webhook:
        :param mock_get_ping_target:
        :type mock_get_ping_target:
        :return:
        :rtype:
        """

        form_data = {
            "announcement_target": "nonexistent",
            "announcement_channel": 1,
            "announcement_text": "Test announcement",
        }

        mock_get_ping_target.side_effect = PingTarget.DoesNotExist
        mock_get_webhook.return_value.url = "http://example.com/webhook"

        context = get_announcement_context_from_form_data(form_data)

        self.assertIsNone(context["announcement_target"]["group_id"])
        self.assertIsNone(context["announcement_target"]["group_name"])
        self.assertEqual(context["announcement_target"]["at_mention"], "")
        self.assertEqual(
            context["announcement_channel"]["webhook"], "http://example.com/webhook"
        )
        self.assertEqual(context["announcement_text"], "Test announcement")

    @patch("aa_discord_announcements.models.PingTarget.objects.get")
    @patch("aa_discord_announcements.models.Webhook.objects.get")
    def test_handles_nonexistent_webhook(self, mock_get_webhook, mock_get_ping_target):
        """
        Test the get_announcement_context_from_form_data function with a nonexistent webhook

        :param mock_get_webhook:
        :type mock_get_webhook:
        :param mock_get_ping_target:
        :type mock_get_ping_target:
        :return:
        :rtype:
        """

        form_data = {
            "announcement_target": "@here",
            "announcement_channel": "nonexistent",
            "announcement_text": "Test announcement",
        }

        mock_get_webhook.side_effect = Webhook.DoesNotExist

        context = get_announcement_context_from_form_data(form_data)

        self.assertEqual(context["announcement_target"]["at_mention"], "@here")
        self.assertIsNone(context["announcement_channel"]["webhook"])
        self.assertEqual(context["announcement_text"], "Test announcement")


class TestWebhookAnnouncementContext(TestCase):
    """
    Test the get_webhook_announcement_context function
    """

    def test_returns_correct_context_with_group_id(self):
        """
        Test the get_webhook_announcement_context function with a group ID

        :return:
        :rtype:
        """

        announcement_context = {
            "announcement_target": {"group_id": 123456789},
            "announcement_text": "Test announcement",
        }

        result = get_webhook_announcement_context(announcement_context)

        self.assertEqual(result["content"], "<@&123456789>\n\nTest announcement")

    def test_returns_correct_context_with_at_here(self):
        """
        Test the get_webhook_announcement_context function with an @here mention

        :return:
        :rtype:
        """

        announcement_context = {
            "announcement_target": {"group_id": None, "at_mention": "@here"},
            "announcement_text": "Test announcement",
        }

        result = get_webhook_announcement_context(announcement_context)

        self.assertEqual(result["content"], "@here\n\nTest announcement")

    def test_handles_empty_announcement_text(self):
        """
        Test the get_webhook_announcement_context function with an empty announcement text

        :return:
        :rtype:
        """

        announcement_context = {
            "announcement_target": {"group_id": 123456789, "at_mention": ""},
            "announcement_text": "",
        }

        result = get_webhook_announcement_context(announcement_context)

        self.assertEqual(result["content"], "<@&123456789>")

    def test_handles_empty_announcement_target(self):
        """
        Test the get_webhook_announcement_context function with an empty announcement target

        :return:
        :rtype:
        """

        announcement_context = {
            "announcement_target": {"group_id": None, "at_mention": ""},
            "announcement_text": "Test announcement",
        }

        result = get_webhook_announcement_context(announcement_context)

        self.assertEqual(result["content"], "\n\nTest announcement")

    def test_handles_empty_context(self):
        """
        Test the get_webhook_announcement_context function with an empty context

        :return:
        :rtype:
        """

        announcement_context = {
            "announcement_target": {"group_id": None, "at_mention": ""},
            "announcement_text": "",
        }

        result = get_webhook_announcement_context(announcement_context)

        self.assertEqual(result["content"], "")


class TestSendToDiscordWebhook(TestCase):
    """
    Test the send_to_discord_webhook function
    """

    @patch("aa_discord_announcements.helper.discord_webhook.Webhook.execute")
    @patch(
        "aa_discord_announcements.helper.discord_webhook.get_webhook_announcement_context"
    )
    @patch("aa_discord_announcements.helper.discord_webhook.get_user_agent")
    def test_sends_announcement_successfully(
        self, mock_get_user_agent, mock_get_webhook_announcement_context, mock_execute
    ):
        """
        Test the send_to_discord_webhook function sending an announcement successfully

        :param mock_get_user_agent:
        :type mock_get_user_agent:
        :param mock_get_webhook_announcement_context:
        :type mock_get_webhook_announcement_context:
        :param mock_execute:
        :type mock_execute:
        :return:
        :rtype:
        """

        mock_get_user_agent.return_value = "TestUserAgent"
        mock_get_webhook_announcement_context.return_value = {
            "content": "Test announcement content"
        }
        mock_user = Mock(spec=User)
        mock_user.profile.main_character.character_name = "TestCharacter"

        announcement_context = {
            "announcement_channel": {"webhook": "http://example.com/webhook"},
            "announcement_text": "Test announcement",
        }

        send_to_discord_webhook(announcement_context, mock_user)

        # Use regex to match the dynamic timestamp
        expected_content_pattern = re.compile(
            r"Test announcement content\n\n-# _Sent by TestCharacter @ \d{4}-\d{2}-\d{2} \d{2}:\d{2} \(EVE time\)_"
        )
        actual_content = mock_execute.call_args[1]["content"]
        self.assertTrue(expected_content_pattern.match(actual_content))
        mock_execute.assert_called_once_with(
            content=actual_content, wait_for_response=True
        )

    @patch("aa_discord_announcements.helper.discord_webhook.Webhook.execute")
    @patch(
        "aa_discord_announcements.helper.discord_webhook.get_webhook_announcement_context"
    )
    @patch("aa_discord_announcements.helper.discord_webhook.get_user_agent")
    def test_handles_missing_webhook_url(
        self, mock_get_user_agent, mock_get_webhook_announcement_context, mock_execute
    ):
        """
        Test the send_to_discord_webhook function handles a missing webhook URL

        :param mock_get_user_agent:
        :type mock_get_user_agent:
        :param mock_get_webhook_announcement_context:
        :type mock_get_webhook_announcement_context:
        :param mock_execute:
        :type mock_execute:
        :return:
        :rtype:
        """

        mock_get_user_agent.return_value = "TestUserAgent"
        mock_get_webhook_announcement_context.return_value = {
            "content": "Test announcement content"
        }
        mock_user = Mock(spec=User)
        mock_user.profile.main_character.character_name = "TestCharacter"

        announcement_context = {
            "announcement_channel": {"webhook": None},
            "announcement_text": "Test announcement",
        }

        with self.assertRaises(ValueError):
            send_to_discord_webhook(announcement_context, mock_user)

    @patch("aa_discord_announcements.helper.discord_webhook.Webhook.execute")
    @patch(
        "aa_discord_announcements.helper.discord_webhook.get_webhook_announcement_context"
    )
    @patch("aa_discord_announcements.helper.discord_webhook.get_user_agent")
    def handles_empty_announcement_text(
        self, mock_get_user_agent, mock_get_webhook_announcement_context, mock_execute
    ):
        """
        Test the send_to_discord_webhook function handles an empty announcement text

        :param mock_get_user_agent:
        :type mock_get_user_agent:
        :param mock_get_webhook_announcement_context:
        :type mock_get_webhook_announcement_context:
        :param mock_execute:
        :type mock_execute:
        :return:
        :rtype:
        """

        mock_get_user_agent.return_value = "TestUserAgent"
        mock_get_webhook_announcement_context.return_value = {"content": ""}
        mock_user = Mock(spec=User)
        mock_user.profile.main_character.character_name = "TestCharacter"

        announcement_context = {
            "announcement_channel": {"webhook": "http://example.com/webhook"},
            "announcement_text": "",
        }

        send_to_discord_webhook(announcement_context, mock_user)

        mock_execute.assert_called_once_with(
            content="\n\n-# _Sent by TestCharacter @ 2023-10-10 10:10 (EVE time)_",
            wait_for_response=True,
        )



"""
Constants
"""

# Standard Library
import os

APP_NAME = "aa-discord-announcements"
PACKAGE_NAME = "aa_discord_announcements"
GITHUB_URL = f"https://github.com/ppfeufer/{APP_NAME}"

DISCORD_WEBHOOK_REGEX = r"https:\/\/discord\.com\/api\/webhooks\/[\d]+\/[a-zA-Z0-9_-]+$"

# All internal URLs need to start with this prefix
INTERNAL_URL_PREFIX = "-"

# aa-discord-announcements/aa_discord_announcements
APP_BASE_DIR = os.path.join(os.path.dirname(__file__))
# aa-discord-announcements/aa_discord_announcements/static/aa_discord_announcements
APP_STATIC_DIR = os.path.join(APP_BASE_DIR, "static", PACKAGE_NAME)



"""
The models
"""

# Standard Library
import re

# Third Party
from requests.exceptions import HTTPError

# Django
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA Discord Announcements
from aa_discord_announcements import __title__
from aa_discord_announcements.app_settings import discord_service_installed
from aa_discord_announcements.constants import DISCORD_WEBHOOK_REGEX

logger = LoggerAddTag(get_extension_logger(__name__), __title__)

if discord_service_installed():
    # Alliance Auth
    from allianceauth.services.modules.discord.models import DiscordUser


def _get_discord_group_info(ping_target: Group) -> dict:
    """
    Get Discord group info or raise an error
    :param ping_target:
    :type ping_target:
    :return:
    :rtype:
    """

    logger.debug("Checking if Discord group info is available for %s", ping_target)

    if not discord_service_installed():
        raise ValidationError(
            message=_("You might want to install the Discord service first …")
        )

    try:
        discord_group_info = DiscordUser.objects.group_to_role(  # pylint: disable=possibly-used-before-assignment
            group=ping_target
        )
    except HTTPError as http_error:
        raise ValidationError(
            message=_(
                "Are you sure you have your Discord linked to your Alliance Auth?"
            )
        ) from http_error

    if not discord_group_info:
        raise ValidationError(
            message=_("This group has not been synced to Discord yet.")
        )

    return discord_group_info


class General(models.Model):
    """
    Meta model for app permissions
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        General :: Meta
        """

        managed = False
        default_permissions = ()
        permissions = (("basic_access", _("Can access this app")),)


class PingTarget(models.Model):
    """
    Discord Ping Targets
    """

    # Discord group to ping
    name = models.OneToOneField(
        to=Group,
        related_name="discord_announcement_pingtarget",
        on_delete=models.CASCADE,
        unique=True,
        verbose_name=_("Group name"),
        help_text=(
            _(
                "Name of the Discord role to ping. "
                "(Note: This must be an Auth group that is synced to Discord.)"
            )
        ),
    )

    # Discord group id
    discord_id = models.CharField(
        max_length=255,
        unique=True,
        blank=True,
        verbose_name=_("Discord ID"),
        help_text=_("ID of the Discord role to ping"),
    )

    # Restrictions
    restricted_to_group = models.ManyToManyField(
        to=Group,
        blank=True,
        related_name="discord_announcement_pingtarget_required_groups",
        verbose_name=_("Group restrictions"),
        help_text=_("Restrict ping rights to the following groups …"),
    )

    # Notes
    notes = models.TextField(
        default="",
        blank=True,
        verbose_name=_("Notes"),
        help_text=_("You can add notes about this configuration here if you want"),
    )

    # Is this group active?
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        verbose_name=_("Is enabled"),
        help_text=_("Whether this ping target is enabled or not"),
    )

    class Meta:  # pylint: disable=too-few-public-methods
        """
        DiscordPingTargets :: Meta
        """

        verbose_name = _("Discord ping target")
        verbose_name_plural = _("Discord ping targets")
        default_permissions = ()

    def __str__(self) -> str:
        return str(self.name)

    def save(
        self,
        force_insert=False,  # pylint: disable=unused-argument
        force_update=False,  # pylint: disable=unused-argument
        using=None,  # pylint: disable=unused-argument
        update_fields=None,  # pylint: disable=unused-argument
    ):
        """
        Add the Discord group ID (if Discord service is active) and save the whole thing
        """

        # Check if the Discord service is active
        if not discord_service_installed():
            raise ValidationError("Discord service is not installed")

        discord_group_info = _get_discord_group_info(ping_target=self.name)
        self.discord_id = discord_group_info["id"]

        super().save()  # Call the "real" save() method.

    def clean(self):
        """
        Check if the group has already been synced to Discord,
        if not, raise an error
        """

        _get_discord_group_info(ping_target=self.name)

        super().clean()


class Webhook(models.Model):
    """
    A Discord webhook
    """

    # Channel name
    name = models.CharField(
        max_length=255,
        unique=True,
        verbose_name=_("Discord channel"),
        help_text=_("Name of the channel this webhook posts to"),
    )

    # Wehbook url
    url = models.CharField(
        max_length=255,
        unique=True,
        verbose_name=_("Webhook URL"),
        help_text=(
            _(
                "URL of this webhook, e.g. "
                "https://discord.com/api/webhooks/123456/abcdef"
            )
        ),
    )

    # Restrictions
    restricted_to_group = models.ManyToManyField(
        to=Group,
        blank=True,
        related_name="discord_announcement_webhook_required_groups",
        verbose_name=_("Group restrictions"),
        help_text=_("Restrict ping rights to the following groups …"),
    )

    # Webhook notes
    notes = models.TextField(
        default="",
        blank=True,
        verbose_name=_("Notes"),
        help_text=_("You can add notes about this webhook here if you want"),
    )

    # Is it enabled?
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        verbose_name=_("Is enabled"),
        help_text=_("Whether this webhook is active or not"),
    )

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Webhook :: Meta
        """

        verbose_name = _("Webhook")
        verbose_name_plural = _("Webhooks")
        default_permissions = ()

    def __str__(self) -> str:
        return str(self.name)

    def clean(self):
        """
        Check if the webhook URL is valid
        :return:
        """

        # Check if it's a valid Discord webhook URL
        if not re.match(pattern=DISCORD_WEBHOOK_REGEX, string=self.url):
            raise ValidationError(
                message=_(
                    "Invalid webhook URL. The webhook URL you entered does not match "
                    "any known format for a Discord webhook. Please check the "
                    "webhook URL."
                )
            )

        super().clean()



"""
Tests for the template tags
"""

# Django
from django.template import Context, Template
from django.test import TestCase, override_settings

# AA Discord Announcements
from aa_discord_announcements import __version__
from aa_discord_announcements.constants import PACKAGE_NAME
from aa_discord_announcements.helper.static_files import calculate_integrity_hash


class TestVersionedStatic(TestCase):
    """
    Test aa_discord_announcements_static
    """

    @override_settings(DEBUG=False)
    def test_versioned_static(self):
        """
        Test versioned static template tag

        :return:
        """

        context = Context(dict_={"version": __version__})
        template_to_render = Template(
            template_string=(
                "{% load aa_discord_announcements %}"
                "{% aa_discord_announcements_static 'css/aa-discord-announcements.min.css' %}"
                "{% aa_discord_announcements_static 'javascript/aa-discord-announcements.min.js' %}"
            )
        )

        rendered_template = template_to_render.render(context=context)

        expected_static_css_src = f'/static/{PACKAGE_NAME}/css/aa-discord-announcements.min.css?v={context["version"]}'
        expected_static_css_src_integrity = calculate_integrity_hash(
            "css/aa-discord-announcements.min.css"
        )
        expected_static_js_src = f'/static/{PACKAGE_NAME}/javascript/aa-discord-announcements.min.js?v={context["version"]}'
        expected_static_js_src_integrity = calculate_integrity_hash(
            "javascript/aa-discord-announcements.min.js"
        )

        self.assertIn(member=expected_static_css_src, container=rendered_template)
        self.assertIn(
            member=expected_static_css_src_integrity, container=rendered_template
        )
        self.assertIn(member=expected_static_js_src, container=rendered_template)
        self.assertIn(
            member=expected_static_js_src_integrity, container=rendered_template
        )

    @override_settings(DEBUG=True)
    def test_versioned_static_with_debug_enabled(self) -> None:
        """
        Test versioned static template tag with DEBUG enabled

        :return:
        :rtype:
        """

        context = Context({"version": __version__})
        template_to_render = Template(
            template_string=(
                "{% load aa_discord_announcements %}"
                "{% aa_discord_announcements_static 'css/aa-discord-announcements.min.css' %}"
            )
        )

        rendered_template = template_to_render.render(context=context)

        expected_static_css_src = f'/static/{PACKAGE_NAME}/css/aa-discord-announcements.min.css?v={context["version"]}'

        self.assertIn(member=expected_static_css_src, container=rendered_template)
        self.assertNotIn(member="integrity=", container=rendered_template)

    @override_settings(DEBUG=False)
    def test_invalid_file_type(self) -> None:
        """
        Test should raise a ValueError for an invalid file type

        :return:
        :rtype:
        """

        context = Context({"version": __version__})
        template_to_render = Template(
            template_string=(
                "{% load aa_discord_announcements %}"
                "{% aa_discord_announcements_static 'invalid/invalid.txt' %}"
            )
        )

        with self.assertRaises(ValueError):
            template_to_render.render(context=context)



"""
App Settings
"""

# Standard Library
from re import RegexFlag

# Django
from django.apps import apps
from django.conf import settings


def discord_service_installed() -> bool:
    """
    Check if the Discord service is installed
    :return: bool
    """

    return apps.is_installed(app_name="allianceauth.services.modules.discord")


def debug_enabled() -> RegexFlag:
    """
    Check if DEBUG is enabled

    :return:
    :rtype:
    """

    return settings.DEBUG



"""
Settings for the admin backend
"""

# Django
from django.contrib import admin
from django.utils.translation import gettext_lazy as _

# AA Discord Announcements
from aa_discord_announcements.models import PingTarget, Webhook


@admin.register(PingTarget)
class PingTargetAdmin(admin.ModelAdmin):
    """
    DiscordPingTargetsAdmin
    """

    list_display = (
        "_name",
        "discord_id",
        "_restricted_to_group",
        "notes",
        "is_enabled",
    )

    filter_horizontal = ("restricted_to_group",)
    readonly_fields = ("discord_id",)
    ordering = ("name",)

    @admin.display(description=_("Group name"), ordering="name")
    def _name(self, obj):
        return obj.name.name

    @admin.display(
        description=_("Group restrictions"), ordering="restricted_to_group__name"
    )
    def _restricted_to_group(self, obj):
        names = [x.name for x in obj.restricted_to_group.all().order_by("name")]

        return ", ".join(names) if names else None


@admin.register(Webhook)
class WebhookAdmin(admin.ModelAdmin):
    """
    WebhookAdmin
    """

    list_display = ("_name", "_url", "_restricted_to_group", "notes", "is_enabled")

    filter_horizontal = ("restricted_to_group",)
    ordering = ("name",)

    @admin.display(description=_("Discord channel"), ordering="name")
    def _name(self, obj):
        return obj.name

    @admin.display(description=_("Webhook URL"), ordering="url")
    def _url(self, obj):
        return obj.url

    @admin.display(
        description=_("Group restrictions"), ordering="restricted_to_group__name"
    )
    def _restricted_to_group(self, obj):
        names = [x.name for x in obj.restricted_to_group.all().order_by("name")]

        return ", ".join(names) if names else None



"""
Helper functions for static integrity calculations
"""

# Standard Library
import os
from pathlib import Path

# Third Party
from sri import Algorithm, calculate_integrity

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA Discord Announcements
from aa_discord_announcements import __title__
from aa_discord_announcements.constants import APP_STATIC_DIR

logger = LoggerAddTag(my_logger=get_extension_logger(__name__), prefix=__title__)


def calculate_integrity_hash(relative_file_path: str) -> str:
    """
    Calculates the integrity hash for a given static file

    :param self:
    :type self:
    :param relative_file_path: The file path relative to the `{APP_NAME}/{PACKAGE_NAME}/static/{PACKAGE_NAME}` folder
    :type relative_file_path: str
    :return: The integrity hash
    :rtype: str
    """

    return calculate_integrity(
        path=Path(os.path.join(APP_STATIC_DIR, relative_file_path)),
        algorithm=Algorithm.SHA512,
    )



# Generated by Django 4.0.7 on 2022-08-08 11:01

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="General",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "permissions": (("basic_access", "Can access this app"),),
                "managed": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Webhook",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="Name of the channel this webhook posts to",
                        max_length=255,
                        unique=True,
                    ),
                ),
                (
                    "url",
                    models.CharField(
                        help_text="URL of this webhook, e.g. https://discord.com/api/webhooks/123456/abcdef",
                        max_length=255,
                        unique=True,
                    ),
                ),
                (
                    "notes",
                    models.TextField(
                        blank=True,
                        default="",
                        help_text="You can add notes about this webhook here if you want",
                    ),
                ),
                (
                    "is_enabled",
                    models.BooleanField(
                        db_index=True,
                        default=True,
                        help_text="Whether this webhook is active or not",
                    ),
                ),
                (
                    "restricted_to_group",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Restrict ping rights to the following group(s) ...",
                        related_name="discord_announcement_webhook_required_groups",
                        to="auth.group",
                    ),
                ),
            ],
            options={
                "verbose_name": "Webhook",
                "verbose_name_plural": "Webhooks",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="PingTarget",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "discord_id",
                    models.CharField(
                        blank=True,
                        help_text="ID of the Discord role to ping",
                        max_length=255,
                        unique=True,
                    ),
                ),
                (
                    "notes",
                    models.TextField(
                        blank=True,
                        default="",
                        help_text="You can add notes about this configuration here if you want",
                    ),
                ),
                (
                    "is_enabled",
                    models.BooleanField(
                        db_index=True,
                        default=True,
                        help_text="Whether this ping target is enabled or not",
                    ),
                ),
                (
                    "name",
                    models.OneToOneField(
                        help_text="Name of the Discord role to ping. (Note: This must be an Auth group that is synced to Discord.)",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="discord_announcement_pingtarget",
                        to="auth.group",
                    ),
                ),
                (
                    "restricted_to_group",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Restrict ping rights to the following group(s) ...",
                        related_name="discord_announcement_pingtarget_required_groups",
                        to="auth.group",
                    ),
                ),
            ],
            options={
                "verbose_name": "Discord Ping Target",
                "verbose_name_plural": "Discord Ping Targets",
                "default_permissions": (),
            },
        ),
    ]



"""
Pages url config
"""

# Django
from django.urls import include, path

# AA Discord Announcements
from aa_discord_announcements import views
from aa_discord_announcements.constants import INTERNAL_URL_PREFIX

app_name: str = "aa_discord_announcements"

# Ajax URLs
ajax_urls = path(
    route="ajax/",
    view=include(
        [
            path(
                route="get-announcement-targets-for-user/",
                view=views.ajax_get_announcement_targets,
                name="ajax_get_announcement_targets",
            ),
            path(
                route="get-webhooks-for-user/",
                view=views.ajax_get_webhooks,
                name="ajax_get_webhooks",
            ),
            path(
                route="create-announcement/",
                view=views.ajax_create_announcement,
                name="ajax_create_announcement",
            ),
        ]
    ),
)

urlpatterns = [
    path(route="", view=views.index, name="index"),
    # Internal URLs
    path(
        route=f"{INTERNAL_URL_PREFIX}/",
        view=include([ajax_urls]),
    ),
]



# Standard Library
from unittest.mock import patch

# Django
from django.contrib import admin
from django.contrib.auth.models import Group
from django.test import TestCase

# AA Discord Announcements
from aa_discord_announcements.admin import PingTargetAdmin, WebhookAdmin
from aa_discord_announcements.models import PingTarget, Webhook


class TestPingTargetAdmin(TestCase):
    """
    Test the PingTargetAdmin class
    """

    @patch(
        "aa_discord_announcements.models._get_discord_group_info",
        return_value={"id": "123456789"},
    )
    def test_displays_correct_name(self, mock_get_discord_group_info):
        """
        Test that the name is displayed correctly in the admin

        :param mock_get_discord_group_info:
        :type mock_get_discord_group_info:
        :return:
        :rtype:
        """

        group = Group.objects.create(name="Test Group")
        ping_target = PingTarget.objects.create(name=group)
        admin_instance = PingTargetAdmin(PingTarget, admin.site)

        self.assertEqual(admin_instance._name(ping_target), "Test Group")

    @patch(
        "aa_discord_announcements.models._get_discord_group_info",
        return_value={"id": "123456789"},
    )
    def test_displays_correct_group_restrictions(self, mock_get_discord_group_info):
        """
        Test that the group restrictions are displayed correctly in the admin

        :return:
        :rtype:
        """

        group1 = Group.objects.create(name="Group 1")
        group2 = Group.objects.create(name="Group 2")
        ping_target = PingTarget.objects.create(name=group1)
        ping_target.restricted_to_group.add(group2)
        admin_instance = PingTargetAdmin(PingTarget, admin.site)

        self.assertEqual(admin_instance._restricted_to_group(ping_target), "Group 2")

    @patch(
        "aa_discord_announcements.models._get_discord_group_info",
        return_value={"id": "123456789"},
    )
    def test_displays_no_group_restrictions_when_none(
        self, mock_get_discord_group_info
    ):
        """
        Test that no group restrictions are displayed when there are none

        :return:
        :rtype:
        """

        group = Group.objects.create(name="Test Group")
        ping_target = PingTarget.objects.create(name=group)
        admin_instance = PingTargetAdmin(PingTarget, admin.site)

        self.assertIsNone(admin_instance._restricted_to_group(ping_target))


class TestWebhookAdmin(TestCase):
    """
    Test the WebhookAdmin class
    """

    @patch(
        "aa_discord_announcements.models._get_discord_group_info",
        return_value={"id": "123456789"},
    )
    def test_displays_correct_name(self, mock_get_discord_group_info):
        """
        Test that the name is displayed correctly in the admin

        :param mock_get_discord_group_info:
        :type mock_get_discord_group_info:
        :return:
        :rtype:
        """

        group = Group.objects.create(name="Test Group")
        webhook = Webhook.objects.create(
            name="Test Webhook", url="https://discord.com/api/webhooks/123456/abcdef"
        )
        webhook.restricted_to_group.set([group])
        admin_instance = WebhookAdmin(Webhook, admin.site)

        self.assertEqual(admin_instance._name(webhook), "Test Webhook")

    @patch(
        "aa_discord_announcements.models._get_discord_group_info",
        return_value={"id": "123456789"},
    )
    def test_displays_correct_url(self, mock_get_discord_group_info):
        """
        Test that the URL is displayed correctly in the admin

        :param mock_get_discord_group_info:
        :type mock_get_discord_group_info:
        :return:
        :rtype:
        """

        group = Group.objects.create(name="Test Group")
        webhook = Webhook.objects.create(
            name="Test Webhook", url="https://discord.com/api/webhooks/123456/abcdef"
        )
        webhook.restricted_to_group.set([group])
        admin_instance = WebhookAdmin(Webhook, admin.site)

        self.assertEqual(
            admin_instance._url(webhook),
            "https://discord.com/api/webhooks/123456/abcdef",
        )

    @patch(
        "aa_discord_announcements.models._get_discord_group_info",
        return_value={"id": "123456789"},
    )
    def test_displays_correct_group_restrictions(self, mock_get_discord_group_info):
        """
        Test that the group restrictions are displayed correctly in the admin

        :param mock_get_discord_group_info:
        :type mock_get_discord_group_info:
        :return:
        :rtype:
        """

        group1 = Group.objects.create(name="Group 1")
        group2 = Group.objects.create(name="Group 2")
        webhook = Webhook.objects.create(
            name="Test Webhook",
            url="https://discord.com/api/webhooks/123456/abcdef",
        )
        webhook.restricted_to_group.add(group1, group2)
        admin_instance = WebhookAdmin(Webhook, admin.site)

        self.assertEqual(
            admin_instance._restricted_to_group(webhook), "Group 1, Group 2"
        )

    @patch(
        "aa_discord_announcements.models._get_discord_group_info",
        return_value={"id": "123456789"},
    )
    def test_displays_no_group_restrictions_when_none(
        self, mock_get_discord_group_info
    ):
        """
        Test that no group restrictions are displayed when there are none

        :param mock_get_discord_group_info:
        :type mock_get_discord_group_info:
        :return:
        :rtype:
        """

        webhook = Webhook.objects.create(
            name="Test Webhook",
            url="https://discord.com/api/webhooks/123456/abcdef",
        )
        admin_instance = WebhookAdmin(Webhook, admin.site)

        self.assertIsNone(admin_instance._restricted_to_group(webhook))



"""
Versioned static URLs to break browser caches when changing the app version
"""

# Standard Library
import os

# Django
from django.template.defaulttags import register
from django.templatetags.static import static
from django.utils.safestring import mark_safe

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA Discord Announcements
from aa_discord_announcements import __title__, __version__
from aa_discord_announcements.app_settings import debug_enabled
from aa_discord_announcements.constants import PACKAGE_NAME
from aa_discord_announcements.helper.static_files import calculate_integrity_hash

logger = LoggerAddTag(my_logger=get_extension_logger(__name__), prefix=__title__)


@register.simple_tag
def aa_discord_announcements_static(
    relative_file_path: str, script_type: str = None
) -> str | None:
    """
    Versioned static URL

    :param relative_file_path: The file path relative to the `{APP_NAME}/{PACKAGE_NAME}/static/{PACKAGE_NAME}` folder
    :type relative_file_path: str
    :param script_type: The script type
    :type script_type: str
    :return: Versioned static URL
    :rtype: str
    """

    logger.debug(f"Getting versioned static URL for: {relative_file_path}")

    file_type = os.path.splitext(relative_file_path)[1][1:]

    logger.debug(f"File extension: {file_type}")

    # Only support CSS and JS files
    if file_type not in ["css", "js"]:
        raise ValueError(f"Unsupported file type: {file_type}")

    static_file_path = os.path.join(PACKAGE_NAME, relative_file_path)
    static_url = static(static_file_path)

    # Integrity hash calculation only for non-debug mode
    sri_string = (
        f' integrity="{calculate_integrity_hash(relative_file_path)}" crossorigin="anonymous"'
        if not debug_enabled()
        else ""
    )

    # Versioned URL for CSS and JS files
    # Add version query parameter to break browser caches when changing the app version
    # Do not add version query parameter for libs as they are already versioned through their file path
    versioned_url = (
        static_url
        if relative_file_path.startswith("libs/")
        else static_url + "?v=" + __version__
    )

    return_value = None

    # Return the versioned URL with integrity hash for CSS
    if file_type == "css":
        return_value = mark_safe(
            f'<link rel="stylesheet" href="{versioned_url}"{sri_string}>'
        )

    # Return the versioned URL with integrity hash for JS files
    if file_type == "js":
        js_type = f' type="{script_type}"' if script_type else ""

        return_value = mark_safe(
            f'<script{js_type} src="{versioned_url}"{sri_string}></script>'
        )

    return return_value



"""
The forms
"""

# Django
from django import forms
from django.utils.safestring import mark_safe
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _


def _get_mandatory_form_label_text(text: str) -> str:
    """
    Label text for mandatory form fields

    :param text:
    :type text:
    :return:
    :rtype:
    """

    required_text = _("This field is mandatory")
    required_marker = (
        f'<span aria-label="{required_text}" class="form-required-marker">*</span>'
    )

    return mark_safe(
        f'<span class="form-field-required">{text} {required_marker}</span>'
    )


def _get_discord_markdown_hint_text() -> str:
    """
    Get the formatted help text for any field that allows Discord Markdown

    :return:
    """

    discord_helpdesk_url = (
        "https://support.discord.com/hc/en-us/articles/210298617"
        "-Markdown-Text-101-Chat-Formatting-Bold-Italic-Underline-"
    )
    discord_markdown_link = format_lazy(
        '<a href="{url}" target="_blank" rel="noopener noreferer">{text}</a>',
        url=discord_helpdesk_url,
        text=_("Discord Markdown"),
    )
    return format_lazy(
        _("Hint: You can use {discord_markdown_link} to format the text."),
        discord_markdown_link=discord_markdown_link,
    )


class AnnouncementForm(forms.Form):
    """
    Announcement Form
    """

    announcement_target = forms.CharField(
        required=False,
        label=_("Announcement target"),
        widget=forms.Select(choices={}),
        help_text=_("Who do you want to ping?"),
    )
    announcement_channel = forms.CharField(
        required=False,
        label=_("Announcement channel"),
        widget=forms.Select(choices={}),
        help_text=_("Select a channel to send the announcement to automatically."),
    )
    announcement_text = forms.CharField(
        required=True,
        label=_get_mandatory_form_label_text(_("Announcement text")),
        widget=forms.Textarea(
            attrs={
                "rows": 10,
                "cols": 20,
                "input_type": "textarea",
                "placeholder": _("Your announcement …"),
            }
        ),
        help_text=_get_discord_markdown_hint_text(),
    )



"""
App config
"""

# Django
from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _

# AA Discord Announcements
from aa_discord_announcements import __version__


class AaDiscordAnnouncementsConfig(AppConfig):
    """
    Application config
    """

    name = "aa_discord_announcements"
    label = "aa_discord_announcements"
    # Translators: This is the app name and version, which will appear in the Django Backend
    verbose_name = _(f"Discord Announcements v{__version__}")



"""
Test checks for installed modules we might use
"""

# Django
from django.test import TestCase, modify_settings

# AA Discord Announcements
from aa_discord_announcements.app_settings import discord_service_installed


class TestModulesInstalled(TestCase):
    """
    Test for installed modules
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()

    @modify_settings(INSTALLED_APPS={"remove": "allianceauth.services.modules.discord"})
    def test_for_discord_service_installed_when_not_installed(self):
        """
        Test for discord_service_installed when it is not
        :return:
        """

        self.assertFalse(expr=discord_service_installed())

    @modify_settings(INSTALLED_APPS={"append": "allianceauth.services.modules.discord"})
    def test_for_discord_service_installed_when_installed(self):
        """
        Test for discord_service_installed when it is installed
        :return:
        """

        self.assertTrue(expr=discord_service_installed())



"""
Handling announcement context data
"""

# AA Discord Announcements
from aa_discord_announcements.models import PingTarget, Webhook


def get_announcement_context_from_form_data(form_data: dict) -> dict:
    """
    Getting announcement context from form data

    :param form_data: The form data from the announcement form
    :type form_data: dict
    :return: The announcement context
    :rtype: dict
    """

    announcement_target_group_id = None
    announcement_target_group_name = None
    announcement_target_at_mention = None

    announcement_target = form_data.get("announcement_target")

    if announcement_target:
        if announcement_target in ["@here", "@everyone"]:
            announcement_target_at_mention = str(announcement_target)
        else:
            try:
                # Check if we deal with a custom announcement target
                target = PingTarget.objects.get(discord_id=announcement_target)
                announcement_target_group_id = int(target.discord_id)
                announcement_target_group_name = str(target.name)
                announcement_target_at_mention = (
                    str(target.name)
                    if str(target.name).startswith("@")
                    else f"@{target.name}"
                )
            except PingTarget.DoesNotExist:
                pass

    # Check for webhooks
    announcement_channel_webhook = None
    announcement_channel = form_data.get("announcement_channel")

    if announcement_channel:
        try:
            channel = Webhook.objects.get(pk=announcement_channel)
            announcement_channel_webhook = channel.url
        except Webhook.DoesNotExist:
            pass

    announcement_context = {
        "announcement_target": {
            "group_id": announcement_target_group_id,
            "group_name": announcement_target_group_name,
            "at_mention": announcement_target_at_mention or "",
        },
        "announcement_channel": {"webhook": announcement_channel_webhook},
        "announcement_text": str(form_data.get("announcement_text", "")),
    }

    return announcement_context


def get_webhook_announcement_context(announcement_context: dict) -> dict:
    """
    Getting the webhook announcement context

    :param announcement_context: The announcement context
    :type announcement_context: dict
    :return: The webhook announcement context
    :rtype: dict
    """

    webhook_announcement_text_content = ""
    webhook_announcement_text_footer = ""
    webhook_announcement_target = ""

    # Ping target
    announcement_target_at_mention = (
        f'<@&{announcement_context["announcement_target"]["group_id"]}>'
        if announcement_context["announcement_target"]["group_id"]
        else f'{announcement_context["announcement_target"]["at_mention"]}'
    )

    if announcement_target_at_mention:
        webhook_announcement_text_content += announcement_target_at_mention

    if announcement_context["announcement_text"]:
        webhook_announcement_text_content += (
            f'\n\n{announcement_context["announcement_text"]}'
        )

    return {
        "target": webhook_announcement_target,
        "content": webhook_announcement_text_content,
        "footer": webhook_announcement_text_footer,
    }
